
import android.app.Activity;
import android.app.Service;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.IBinder;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;

import com.viatom.azur.bluetooth.BTConnectListener;
import com.viatom.azur.bluetooth.BTUtils;
import com.viatom.azur.bluetooth.GetInfoThreadListener;
import com.viatom.azur.bluetooth.PingThreadListener;
import com.viatom.azur.bluetooth.ReadFileListener;
import com.viatom.azur.measurement.ECGInnerItem;
import com.viatom.azur.measurement.ECGItem;
import com.viatom.azur.measurement.SPO2Item;
import com.viatom.azur.measurement.TempItem;
import com.viatom.azur.utils.FileDriver;
import com.viatom.azur.utils.FileUtils;
import com.viatom.azur.utils.LogUtils;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.io.File;
import android.os.Environment;

import java.nio.charset.StandardCharsets;
import java.util.Stack;

public class BTActivity extends Activity implements BTConnectListener, ServiceConnection, PingThreadListener, GetInfoThreadListener{


    private LinkedList<String> objectives = new LinkedList<>();
    private Intent outputIntent = new Intent();
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_bt);
        Intent i = new Intent(this, BTUtils.class);
        bindService(i, this, Service.BIND_AUTO_CREATE);
        objectives.add("ecg.dat");
        objectives.add("oxi.dat");
        objectives.add("tmp.dat");
        //objectives.add("slm.dat");
        objectives.add("bpcal.dat");
    }

    public class Worker implements Runnable{

        private BTUtils.BTBinder mBinder;
        private String objective;

        public Worker(BTUtils.BTBinder binder, String objective){
            mBinder = binder;
            this.objective = objective;
        }

        public void setBinder(BTUtils.BTBinder binder){
            mBinder = binder;
        }

        public void setObjective(String objective){
            this.objective = objective;
        }

        @Override
        public void run(){
            mBinder.interfaceReadFile(objective, (byte)0,5000, new ReadFileListenerImpl());
            while(mBinder.isAnyThreadRunning()){
                if(!mBinder.isAnyThreadRunning()){
                    break;
                }
            }
        }

    }

    public class ReadFileListenerImpl implements ReadFileListener{
        @Override
        public void onReadPartFinished(String fileName, byte fileType, float percentage){
            LogUtils.d(fileName + "Read " + percentage*100 + "%");
        }
        @Override
        public void onReadSuccess(String fileName, byte fileType, byte[] fileBuf){
            String location = Environment.getExternalStorageDirectory().toString() + "/CheckMeIM";
            File f = new File(location);
            if(!f.exists()){
                f.mkdirs();
            }
            //FileDriver.write(f , fileName, fileBuf);
            FileUtils.saveListToFileWithoutUndownloaded(f, fileName, null);
        }
        @Override
        public void onReadFailed(String fileName, byte fileType, byte errCode){
            Log.e("ERROR",fileName + " : Read Failed - No measurement detected, proceeding if there are any objectives left.");
        }
    }

    private BTUtils.BTBinder mBinder = null;
    @Override
    public void onGetInfoFailed(byte errCode){
        Log.e("ERR", "INFO GET FAILED");
    }

    @Override
    public void onGetInfoSuccess(String info){
        tesRead();
    }

    @Override
    public void onPingFailed(byte errCode){
    }

    @Override
    public void onPingSuccess(){
        mBinder.interfaceGetInfo(10000, this);
    }

    @Override
    public void onConnectFailed(byte errCode){
        Log.e("ERR", "CONNECTION FAILED : " + errCode);
    }

    @Override
    public void onConnectSuccess(){
        Log.e("CHECKME" , "CONNECT SUCCESS");
        mBinder.interfacePing(1000, this);
    }
        @Override
        public void onServiceConnected(ComponentName componentName, IBinder service) {
            mBinder = (BTUtils.BTBinder) service;
            Log.d("BT BIND SERVICE","BOUND");
            testConnectCheckme("B0:B4:48:94:99:41");
        }
        @Override
        public void onServiceDisconnected(ComponentName componentName) {
        }
        public void testConnectCheckme(String MAC){
            mBinder.interfaceConnect(MAC,this);
        }
   public void tesRead()
   {
       //mBinder.interfaceReadFile("ecg.dat", (byte)0,5000, this);
       //mBinder.interfaceReadFile(objectives.get(0), (byte)0,5000, new ReadFileListenerImpl());

       Worker workerECG = new Worker(mBinder, "ecg.dat");
       Thread t = new Thread(workerECG);
       t.start();
       try{
           t.join();
           Worker workerOXI = new Worker(mBinder, "oxi.dat");
           t = new Thread(workerOXI);
           t.start();

           t.join();
           Worker workerTMP = new Worker(mBinder, "tmp.dat");
           t = new Thread(workerTMP);
           t.start();

           t.join();
           Worker workerBPCAL = new Worker(mBinder, "bpcal.dat");
           t = new Thread(workerBPCAL);
           t.start();

           t.join();
           Worker workerSLM = new Worker(mBinder, "slm.dat");
           t = new Thread(workerSLM);
           t.start();

      }catch (InterruptedException e){

       }

       String location = Environment.getExternalStorageDirectory().toString() + "/CheckMeIM";

//       List<ECGItem> ECGItems = FileUtils.readECGList(new File(location), "ecg.dat");
//        if(ECGItems != null && ECGItems.size() != 0){
//            ECGItem item = ECGItems.get(0);
//            String fileName = new SimpleDateFormat("yyyyMMddHHmmss").format(item.getDate());
//            ECGInnerItem innerItem = FileUtils.readECGInnerItem(new File(location), fileName);
//            item.setInnerItem(innerItem);
//            Log.e("DEBUG", "HR = "+innerItem.getHR());
//        }


       Log.e("OXI ITEM", "TRYING TO READ OXI VALUES");
       List<SPO2Item> SPO2Items = FileUtils.readSPO2List(new File(location), "oxi.dat");
       if(SPO2Items != null && SPO2Items.size() != 0){
           for(int i = 0; i < SPO2Items.size(); i++){
               Log.e("OXI ITEM", SPO2Items.get(i).getOxygen() + " OXI");
               Log.e("OXI ITEM", SPO2Items.get(i).getPi()+" PI");
               Log.e("OXI ITEM", SPO2Items.get(i).getPr()+" PR");
           }
       }


       //
//       Log.e("TEMP ITEM", "TRYING TO READ TEMP VALUES");
//       List<TempItem> TEMPItems = FileUtils.readTempList(new File(location), "tmp.dat");
//       if(TEMPItems != null && TEMPItems.size() != 0){
//           for(int i = 0; i < TEMPItems.size(); i++){
//               Log.e("TEMP ITEM", TEMPItems.get(i).getResult()+"");
//           }
//       }



//      while(mBinder.isAnyThreadRunning()){
//          continue;
//      }
//       mBinder.interfaceReadFile("oxi.dat", (byte)0,5000, this);
//       while(mBinder.isAnyThreadRunning()){
//           continue;
//       }
//       mBinder.interfaceReadFile("ecg.dat", (byte)0,5000, this);
//       while(mBinder.isAnyThreadRunning()){
//           continue;
//       }
//       mBinder.interfaceReadFile("slm.dat", (byte)0,5000, this);
//       while(mBinder.isAnyThreadRunning()){
//           continue;
//       }
//       mBinder.interfaceReadFile("bpcal.dat", (byte)0,5000, this);
   }

    public void writeFile(){
        String filename = "myfile";
        String string = "Hello world!";
        FileOutputStream outputStream;

        try {
            outputStream = openFileOutput(filename, Context.MODE_APPEND);
            outputStream.write(string.getBytes());
            outputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void testReadFile(){
        //List<ECGItem> ecgItems = FileUtils.readECGList(new File("/sdcard0/CheckmeMobile"), "ecg.dat");
//        if(ecgItems != null && ecgItems.size() != 0){
//            ECGItem ecgItem = ecgItems.get(0);
//            String fileName = new SimpleDateFormat("yyyyMMddHHmmss").format(ecgItem.getDate());
//            ECGInnerItem innerItem = FileUtils.readECGInnerItem(new File("/sdcard0/CheckmeMobile"), fileName);
//            ecgItem.setInnerItem(innerItem);
//
//            Log.e("DEBUG - Heartrate", innerItem.getHR() + "");
//            File f = new File("/sdcard0/CheckmeMobile/hello.dat");
//
//            Log.e("DEBUG", f.getAbsoluteFile()+"");
//        }
    }




    public ECGItem testDecodeECGItem(byte[] buf){
        if(buf != null){
            ECGItem item = new ECGItem(buf);
            Log.e("DEBUG ",item.getInnerItem() + "");
            return item;
        }
        return null;
    }

    public void finishIntent(){
        setResult(RESULT_OK, outputIntent);
        finish();
    }


}
